@@ -1,13 +1,13 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2019 by Dominique Dumont.
+# This software is Copyright (c) 2005-2022 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-package Config::Model::Backend::PlainFile 2.138;
+package Config::Model::Backend::PlainFile 2.153;
 
 use 5.10.1;
 use Carp;
@@ -33,6 +33,13 @@ sub annotation { return 0; }
 # file_path is undef
 sub skip_open { 1; }
 
+sub get_file_name {
+    my ($self, %args) = @_;
+
+    my $obj = $args{object}->fetch_element( name => $args{elt} );
+    return $args{file} ? $obj->compute_string($args{file}) : $args{elt};
+}
+
 sub read {
     my $self = shift;
     my %args = @_;
@@ -54,11 +61,11 @@ sub read {
     foreach my $elt ( $node->get_element_names(all => 1) ) {
         my $obj = $args{object}->fetch_element( name => $elt );
 
-        my $file_name = $args{file} ? $obj->compute_string($args{file}) : $elt;
+        my $file_name = $self->get_file_name(%args, elt => $elt);
         my $dir = $self->get_tuned_config_dir(%args);
         my $file = $dir->child($file_name);
 
-        $logger->trace("looking for plainfile $file ");
+        $logger->trace("looking for plainfile $file for ", $obj->location);
 
         my $type = $obj->get_type;
 
@@ -134,7 +141,7 @@ sub write {
     foreach my $elt ( $node->get_element_name() ) {
         my $obj = $args{object}->fetch_element( name => $elt );
 
-        my $file_name = $args{file} ? $obj->compute_string($args{file}) : $elt;
+        my $file_name = $self->get_file_name(%args, elt => $elt);
         my $file = $dir->child($file_name);
 
         my $type = $obj->get_type;
@@ -187,7 +194,8 @@ sub delete {
     foreach my $elt ( $node->get_element_name() ) {
         my $obj = $node->fetch_element( name => $elt );
 
-        my $file = $dir->child($args{file} ? $obj->compute_string($args{file}) : $elt);
+        my $file_name = $self->get_file_name(%args, elt => $elt);
+        my $file = $dir->child( $file_name );
         $logger->info( "Removing $file (deleted node)" );
         $file->remove;
     }
@@ -212,7 +220,7 @@ Config::Model::Backend::PlainFile - Read and write config as plain file
 
 =head1 VERSION
 
-version 2.138
+version 2.153
 
 =head1 SYNOPSIS
 
@@ -340,7 +348,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2019 by Dominique Dumont.
+This software is Copyright (c) 2005-2022 by Dominique Dumont.
 
 This is free software, licensed under:
 
