@@ -1,13 +1,13 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2021 by Dominique Dumont.
+# This software is Copyright (c) 2005-2022 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-package Config::Model::BackendMgr 2.141;
+package Config::Model::BackendMgr 2.153;
 
 use Mouse;
 use strict;
@@ -112,9 +112,12 @@ sub get_cfg_file_path {
     # config file override
     my $cfo = $args{config_file};
 
-    if ( defined $cfo ) {
-        my $root_path = $args{root} // path('.');
-        my $override =  $root_path->child($cfo);
+    if ( defined $cfo) {
+        my $override
+            = $args{root}   ? $args{root}->child($cfo)
+            : $cfo =~ m!^/! ? path($cfo)
+            :                 path('.')->child($cfo);
+
         my $mode = $w ? 'write' : 'read';
         $logger->trace("$args{backend} override target file is $override ($mode mode)");
         return ( 1, $override );
@@ -324,7 +327,6 @@ sub try_read_backend {
         $res = $backend_obj->$f(
             @read_args,
             file_path => $file_path,
-            io_handle => Config::Model::DeprecatedHandle->new($fh),
             object    => $self->node,
         );
     };
@@ -411,7 +413,6 @@ sub auto_write_init {
         # override needed for "save as" button
         my %backend_args = (
             @wr_args,
-            io_handle => Config::Model::DeprecatedHandle->new($fh),
             file_path => $file_path,
             object    => $node,
             %cb_args            # override from user
@@ -435,8 +436,6 @@ sub auto_write_init {
         return defined $res ? $res : $@ ? 0 : 1;
     };
 
-    # FIXME: enhance write back mechanism so that different backend *and* different nodes
-    # work as expected
     $logger->trace( "registering write $backend in node " . $self->node->name );
 
     $instance->register_write_back(  $self->node->location, $backend, $wb  );
@@ -487,7 +486,7 @@ sub close_file_to_write {
     if ($error) {
         # restore backup and display error
         $logger->warn("Error during write, restoring backup data in $file_path" );
-        $file_path->spew_utf8( $self->file_backup );
+        $file_path->append_utf8({ truncate => 1 }, $self->file_backup );
         $error->rethrow if ref($error) and $error->can('rethrow');
         die $error;
     }
@@ -508,34 +507,6 @@ sub is_auto_write_for_type {
 
 __PACKAGE__->meta->make_immutable;
 
-package Config::Model::DeprecatedHandle 2.141;
-
-our $AUTOLOAD;
-
-sub new {
-    my $class = shift;
-    my $fh = shift;
-
-    return defined $fh ? bless \$fh, $class : undef;
-}
-
-sub AUTOLOAD {
-    my $self = shift;
-    my $f = $AUTOLOAD;
-    $f =~ s/.*:://;
-    my ($package, $filename, $line) = caller;
-
-    # $$self may not be defined during destruction
-    if ($$self and $self->can($f)) {
-        $logger->warn(
-            "io_handle backend parameter is deprecated, ",
-            "please use file_path parameter. ",
-            "(called $f at $filename:$line)"
-        ) unless $package eq "Config::Model::BackendMgr";
-
-        $$self->$f(@_);
-    }
-}
 1;
 
 # ABSTRACT: Load configuration node on demand
@@ -552,7 +523,7 @@ Config::Model::BackendMgr - Load configuration node on demand
 
 =head1 VERSION
 
-version 2.141
+version 2.153
 
 =head1 SYNOPSIS
 
@@ -813,7 +784,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2021 by Dominique Dumont.
+This software is Copyright (c) 2005-2022 by Dominique Dumont.
 
 This is free software, licensed under:
 
