@@ -1,18 +1,19 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2021 by Dominique Dumont.
+# This software is Copyright (c) 2005-2022 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-package Config::Model::Value 2.142;
+package Config::Model::Value 2.153;
 
-use 5.10.1;
+use v5.20;
 
 use strict;
 use warnings;
+use feature "switch";
 
 use Mouse;
 use Mouse::Util::TypeConstraints;
@@ -30,8 +31,7 @@ use Scalar::Util qw/weaken/;
 use Carp;
 use Storable qw/dclone/;
 use Path::Tiny;
-use List::MoreUtils qw(any) ;
-
+use List::Util qw(any) ;
 extends qw/Config::Model::AnyThing/;
 
 with "Config::Model::Role::WarpMaster";
@@ -39,6 +39,9 @@ with "Config::Model::Role::Grab";
 with "Config::Model::Role::HelpAsText";
 with "Config::Model::Role::ComputeFunction";
 
+use feature qw/postderef signatures/;
+no warnings qw/experimental::postderef experimental::smartmatch experimental::signatures/;
+
 my $logger        = get_logger("Tree::Element::Value");
 my $user_logger   = get_logger("User");
 my $change_logger = get_logger("Anything::Change");
@@ -110,12 +113,22 @@ has error_list => (
     handles => {
         add_error    => 'push',
         clear_errors => 'clear',
-        error_msg    => [ join => "\n" ],
         has_error    => 'count',
         all_errors   => 'elements',
         is_ok        => 'is_empty'
     } );
 
+sub error_msg  ($self) {
+    my $msg = '';
+    if ($self->has_error) {
+        my @add;
+        push @add, $self->compute_obj->compute_info if $self->compute_obj;
+        push @add, $self->{_migrate_from}->compute_info if $self->{_migrate_from};
+        $msg = join("\n", $self->all_errors, @add);
+    }
+    return $msg;
+}
+
 has warning_list => (
     is      => 'ro',
     isa     => 'ArrayRef',
@@ -131,13 +144,10 @@ has warning_list => (
     } );
 
 # as some information must be backed up even though they are not
-# attributes, we cannot move below code in BUILD.
-around BUILDARGS => sub {
-    my $orig  = shift;
-    my $class = shift;
-    my %args  = @_;
-    my %h     = map { ( $_ => $args{$_} ); } grep { defined $args{$_} } @backup_list;
-    return $class->$orig( backup => dclone( \%h ), @_ );
+# attributes, we cannot move code below in BUILD.
+around BUILDARGS => sub ($orig, $class, %args) {
+    my %h = map { ( $_ => $args{$_} ); } grep { defined $args{$_} } @backup_list;
+    return $class->$orig( backup => dclone( \%h ), %args );
 };
 
 sub BUILD {
@@ -159,14 +169,12 @@ sub BUILD {
     return $self;
 }
 
-override 'needs_check' => sub {
-    my $self = shift;
-
+override 'needs_check' => sub ($self, @args) {
     if ($self->instance->layered) {
         # don't check value and don't store value in layered mode
         return 0;
     }
-    elsif (@_) {
+    elsif (@args) {
         return super();
     }
     else {
@@ -177,9 +185,7 @@ override 'needs_check' => sub {
     }
 };
 
-sub notify_change {
-    my $self       = shift;
-    my %args       = @_;
+around notify_change => sub ($orig, $self, %args) {
     my $check_done = $args{check_done} || 0;
 
     return if $self->instance->initial_load and not $args{really};
@@ -192,14 +198,14 @@ sub notify_change {
 
     $self->needs_check(1) unless $check_done;
     {
-        no warnings 'uninitialized';
         croak "needless change with $args{new}"
             if defined $args{old}
             and defined $args{new}
             and $args{old} eq $args{new};
     }
-    $self->map_write_as_inplace( @args{qw/old new/} );
-    $self->SUPER::notify_change( %args, value_type => $self->value_type );
+    $args{new} = $self->map_write_as( $args{new} );
+    $args{old} = $self->map_write_as( $args{old} );
+    $self->$orig( %args, value_type => $self->value_type );
 
     # shake all warped or computed objects that depends on me
     foreach my $s ( $self->get_depend_slave ) {
@@ -207,7 +213,8 @@ sub notify_change {
             if $change_logger->is_debug;
         $s->needs_check(1);
     }
-}
+    return;
+};
 
 # internal method
 sub set_default {
@@ -242,6 +249,7 @@ sub set_default {
 
         $self->{$item} = $def;
     }
+    return;
 }
 
 # set up relation between objects required by the compute constructor
@@ -290,6 +298,7 @@ sub register_in_other_value {
             $master->register_dependency($self);
         }
     }
+    return;
 }
 
 # internal
@@ -300,10 +309,10 @@ sub perform_compute {
     my $result = $self->compute_obj->compute;
 
     # check if the computed result fits with the constraints of the
-    # Value model
-    my ($ok, $value, $error, $warn) = $self->_check_value(value => $result);
+    # Value model, but don't check if it's mandatory
+    my ($value, $error, $warn) = $self->_check_value(value => $result);
 
-    if ( not $ok ) {
+    if ( scalar $error->@* ) {
         my $error = join("\n", (@$error, $self->compute_info));
 
         Config::Model::Exception::WrongValue->throw(
@@ -313,13 +322,13 @@ sub perform_compute {
     }
 
     $logger->trace("done");
-    return $ok ? $result : undef;
+    return $result;
 }
 
 # internal, used to generate error messages
 sub compute_info {
     my $self = shift;
-    $self->compute_obj->compute_info;
+    return $self->compute_obj->compute_info;
 }
 
 sub set_migrate_from {
@@ -349,6 +358,7 @@ sub set_migrate_from {
 
     # resolve any recursive variables before registration
     my $v = $self->{_migrate_from}->compute_variables;
+    return;
 }
 
 # FIXME: should it be used only once ???
@@ -377,11 +387,9 @@ sub migrate_value {
 
     #print "check result: $ok\n";
     if ( not $ok ) {
-        my $error = $self->error_msg . "\n\t" . $self->{_migrate_from}->compute_info;
-
         Config::Model::Exception::WrongValue->throw(
             object => $self,
-            error  => "migrated value error:\n\t" . $error
+            error  => "migrated value error:\n\t" . $self->error_msg
         );
     }
 
@@ -393,10 +401,8 @@ sub migrate_value {
     return $ok ? $result : undef;
 }
 
-sub setup_enum_choice {
-    my $self = shift;
-
-    my @choice = ref $_[0] ? @{ $_[0] } : @_;
+sub setup_enum_choice ($self, @args) {
+    my @choice = ref $args[0] ? @{ $args[0] } : @args;
 
     $logger->debug( $self->name, " setup_enum_choice with '", join( "','", @choice ), "'" );
 
@@ -416,6 +422,7 @@ sub setup_enum_choice {
             delete $self->{$_};
         }
     }
+    return;
 }
 
 sub setup_match_regexp {
@@ -441,6 +448,7 @@ sub setup_match_regexp {
             error  => "Unvalid $what regexp for '$str': $@"
         );
     }
+    return;
 }
 
 sub check_validation_regexp {
@@ -485,6 +493,7 @@ sub check_validation_regexp {
         ) unless ref $v eq 'HASH';
 
     }
+    return;
 }
 
 sub setup_grammar_check {
@@ -517,19 +526,18 @@ sub setup_grammar_check {
             error  => "Unvalid grammar for '$str': $@"
         );
     }
+    return;
 }
 
 # warning : call to 'set' are not cumulative. Default value are always
 # restored. Lest keeping track of what was modified with 'set' is
 # too confusing.
-sub set_properties {
-    my $self = shift;
-
+sub set_properties ($self, @args) {
     # cleanup all parameters that are handled by warp
     for ( @allowed_warp_params ) { delete $self->{$_} }
 
     # merge data passed to the constructor with data passed to set_properties
-    my %args = ( %{ $self->{backup} }, @_ );
+    my %args = ( %{ $self->backup // {} }, @args );
 
     # these are handled by Node or Warper
     for ( qw/level/ ) { delete $args{$_} }
@@ -602,6 +610,7 @@ sub set_help {
     my ( $self, $args ) = @_;
     return unless defined $args->{help};
     $self->{help} = delete $args->{help};
+    return;
 }
 
 # this code is somewhat dead as warping value_type is no longer supported
@@ -644,6 +653,7 @@ sub set_value_type {
         Config::Model::Exception::Model->throw( object => $self, error => $msg )
             unless defined $self->{warp};
     }
+    return;
 }
 
 
@@ -669,11 +679,11 @@ sub submit_to_refer_to {
     else {
         croak "value's submit_to_refer_to: undefined refer_to or computed_refer_to";
     }
+    return;
 }
 
-sub setup_reference_choice {
-    my $self = shift;
-    $self->setup_enum_choice(@_);
+sub setup_reference_choice ($self, @args) {
+    return $self->setup_enum_choice(@args);
 }
 
 sub reference_object {
@@ -742,7 +752,7 @@ sub get_info {
     my $std = $self->fetch(qw/mode standard check no/);
 
     if ( defined $self->upstream_default ) {
-        push @items, "upstream_default value: " . $self->upstream_default;
+        push @items, "upstream_default value: " . $self->map_write_as( $self->upstream_default );
     }
     elsif ( defined $std ) {
         push @items, "default value: $std";
@@ -769,6 +779,11 @@ sub get_info {
         }
     }
 
+    foreach my $what (qw/write_as/) {
+        my $v = $self->$what();
+        push @items, "$what: @$v" if defined $v;
+    }
+
     return @items ;
 }
 
@@ -813,11 +828,7 @@ sub enum_error {
     return @error;
 }
 
-sub _check_value {
-    my $self = shift;
-    croak "check_value needs a value to check" unless @_ > 1;
-
-    my %args      = @_;
+sub _check_value ($self, %args) {
     my $value     = $args{value};
     my $quiet     = $args{quiet} || 0;
     my $check     = $args{check} || 'yes';
@@ -827,7 +838,6 @@ sub _check_value {
     #croak "Cannot specify a value with fix = 1" if $apply_fix and exists $args{value} ;
 
     if ( $logger->is_debug ) {
-        no warnings 'uninitialized';
         my $v = defined $value ? $value : '<undef>';
         my $loc = $self->location;
         my $msg =
@@ -867,8 +877,8 @@ sub _check_value {
         if (defined $value) {
             my $path = path($value);
             if ($path->exists) {
-                my $check = 'is_'.$vt ;
-                push @warn, "$value is not a $vt" if not path($value)->$check;
+                my $check_sub = 'is_'.$vt ;
+                push @warn, "$value is not a $vt" if not path($value)->$check_sub;
             }
             else {
                 push @warn, "$vt $value does not exists" ;
@@ -922,20 +932,6 @@ sub _check_value {
         Config::Model::Exception::Model->throw( object => $self, message => $msg );
     }
 
-    # a value may be mandatory and have a default value with layers
-    if ( $self->{mandatory}
-         and $check eq 'yes'
-         and ( $mode =~ /backend|user/ )
-         and ( not defined $value or not length($value) )
-         and ( not defined $self->{layered} or not length($self->{layered}))
-        ) {
-        # check only "empty" mode.
-        my $msg = "Undefined mandatory value.";
-        $msg .= $self->warp_error
-            if defined $self->{warped_attribute}{default};
-        push @error, $msg;
-    }
-
     if ( defined $self->{match_regexp} and defined $value ) {
         push @error, "value '$value' does not match regexp " . $self->{match}
             unless $value =~ $self->{match_regexp};
@@ -993,15 +989,36 @@ sub _check_value {
         " errors and ", scalar @warn, " warnings"
     );
 
-    my $ok = not @error;
-    return ($ok, $value, \@error, \@warn);
+    # return $value because it may be modified by apply_fixes
+    return ($value, \@error, \@warn);
 }
 
+sub _check_mandatory_value ($self, %args) {
+    my $value     = $args{value};
+    my $check     = $args{check} || 'yes';
+    my $mode      = $args{mode} || 'backend';
+    my $error     = $args{error} // carp "Missing error parameter";
 
-sub check_value {
-    my $self = shift;
+    # a value may be mandatory and have a default value with layers
+    if ( $self->{mandatory}
+         and $check eq 'yes'
+         and ( $mode =~ /backend|user/ )
+         and ( not defined $value or not length($value) )
+         and ( not defined $self->{layered} or not length($self->{layered}))
+        ) {
+        # check only "empty" mode.
+        my $msg = "Undefined mandatory value.";
+        $msg .= $self->warp_error
+            if defined $self->{warped_attribute}{default};
+        push $error->@*, $msg;
+    }
 
-    my ($ok, $value, $error, $warn) = $self->_check_value(@_);
+    return;
+}
+
+sub check_value ($self, @args) {
+    my ($value, $error, $warn) = $self->_check_value(@args);
+    $self->_check_mandatory_value(@args, value => $value, error => $error);
     $self->clear_errors;
     $self->clear_warnings;
     $self->add_error(@$error)  if @$error;
@@ -1009,6 +1026,7 @@ sub check_value {
 
     $logger->trace("done");
 
+    my $ok = not $error->@*;
     # return $value because it may be updated by apply_fix
     return wantarray ? ($ok, $value) : $ok;
 }
@@ -1036,6 +1054,7 @@ sub run_code_on_value {
         $self->{nb_of_fixes}++ if ( defined $fix and not $apply_fix );
         $self->apply_fix( $fix, $value_r, $msg ) if ( defined $fix and $apply_fix );
     }
+    return;
 }
 
 # function that may be used in eval'ed code to use file in there (in
@@ -1050,7 +1069,7 @@ sub run_code_on_value {
     # use file() function instead of $file->() sub ref
     my $val ;
     sub set_val {
-        $val = shift;
+        return $val = shift;
     }
     sub file {
         return $val->root_path->child(shift);
@@ -1070,6 +1089,7 @@ sub run_code_set_on_value {
 
         my $sub = sub {
             local $_ = shift;
+            ## no critic (TestingAndDebugging::ProhibitNoWarning)
             no warnings "uninitialized";
             my $ret = eval($code); ## no critic (ProhibitStringyEval)
             if ($@) {
@@ -1083,6 +1103,7 @@ sub run_code_set_on_value {
 
         $self->run_code_on_value( $value_r, $apply_fix, $array, $label, $sub, $msg, $fix );
     }
+    return;
 }
 
 sub run_regexp_set_on_value {
@@ -1098,6 +1119,7 @@ sub run_regexp_set_on_value {
         my $fix = $w_info->{$rxp}{fix};
         $self->run_code_on_value( $value_r, $apply_fix, $array, 'regexp', $sub, $msg, $fix );
     }
+    return
 }
 
 sub has_fixes {
@@ -1130,6 +1152,7 @@ sub apply_fixes {
     } while ( $self->{nb_of_fixes} and $old > $new );
 
     $self->show_warnings($self->_fetch_no_check);
+    return;
 }
 
 # internal: called by check when a fix is required
@@ -1154,6 +1177,7 @@ sub apply_fix {
         );
     }
 
+    ## no critic (TestingAndDebugging::ProhibitNoWarning)
     no warnings "uninitialized";
     if ( $_ ne $$value_r ) {
         $fix_logger->info( $self->location . ": fix changed value from '$$value_r' to '$_'" );
@@ -1163,6 +1187,7 @@ sub apply_fix {
     else {
         $fix_logger->info( $self->location . ": fix did not change value '$$value_r'" );
     }
+    return;
 }
 
 sub _store_fix {
@@ -1185,6 +1210,7 @@ sub _store_fix {
         );
     }
 
+    ## no critic (TestingAndDebugging::ProhibitNoWarning)
     no warnings "uninitialized";
     # in case $old is the default value and $new is undef
     if ($old_v ne $new_v) {
@@ -1195,6 +1221,7 @@ sub _store_fix {
         );
         $self->trigger_warp($new_v) if defined $new_v and $self->has_warped_slaves;
     }
+    return;
 }
 
 # read checks should be blocking
@@ -1203,19 +1230,18 @@ sub check {
     goto &check_fetched_value;
 }
 
-sub check_fetched_value {
-    my $self = shift;
-
+sub check_fetched_value ($self, @args) {
     if ( $logger->is_debug ) {
+        ## no critic (TestingAndDebugging::ProhibitNoWarning)
         no warnings 'uninitialized';
         $logger->debug( "called for " . $self->location . " from " . join( ' ', caller ),
-            " with @_" );
+            " with @args" );
     }
 
     my %args =
-          @_ == 0 ? ( value => $self->{data} )
-        : @_ == 1 ? ( value => $_[0] )
-        :           @_;
+          @args == 0 ? ( value => $self->{data} )
+        : @args == 1 ? ( value => $args[0] )
+        :              @args;
 
     my $value = exists $args{value} ? $args{value} : $self->{data};
     my $silent = $args{silent} || 0;
@@ -1239,9 +1265,7 @@ sub check_fetched_value {
     return wantarray ? $self->all_errors : $self->is_ok;
 }
 
-sub show_warnings {
-    my ($self, $value, $silent) = @_ ;
-
+sub show_warnings ($self, $value, $silent = 0) {
     # old_warn is used to avoid warning the user several times for the
     # same reason (i.e. when storing and fetching value). We take care
     # to clean up this hash each time store is run
@@ -1260,24 +1284,19 @@ sub show_warnings {
                 $user_logger->debug($w_msg);
             }
             else {
-                if ($::_use_log4perl_to_warn) {
-                    $user_logger->warn($w_msg);
-                }
-                else {
-                    warn "$w_msg\n";
-                }
+                $user_logger->warn($w_msg);
             }
         }
     }
     $self->{old_warning_hash} = \%warn_h;
+    return;
 }
 
-sub store {
-    my $self = shift;
+sub store ($self, @args) {
     my %args =
-          @_ == 1 ? ( value => $_[0] )
-        : @_ == 3 ? ( 'value', @_ )
-        :           @_;
+          @args == 1 ? ( value => $args[0] )
+        : @args == 3 ? ( 'value', @args )
+        :              @args;
     my $check = $self->_check_check( $args{check} );
     my $silent = $args{silent} || 0;
 
@@ -1296,6 +1315,7 @@ sub store {
 
     my $value = $self->transform_value( value => $incoming_value, check => $check );
 
+    ## no critic (TestingAndDebugging::ProhibitNoWarning)
     no warnings qw/uninitialized/;
     if ($self->instance->initial_load) {
         # may send more than one notification
@@ -1352,6 +1372,7 @@ sub _store_value {
         $self->{preset} = $value;
     }
     else {
+        ## no critic (TestingAndDebugging::ProhibitNoWarning)
         no warnings 'uninitialized';
         my $old = $self->{data} // $self->_fetch_std;
         my $new = $value        // $self->_fetch_std;
@@ -1366,10 +1387,7 @@ sub _store_value {
 }
 
 # this method is overriden in layered Value
-sub _store {
-    my $self = shift;
-    my %args = @_;
-
+sub _store ($self, %args) {
     my ( $value, $check, $silent, $notify_change, $ok ) =
         @args{qw/value check silent notify_change ok/};
 
@@ -1390,18 +1408,12 @@ sub _store {
     else {
         $self->instance->add_error( $self->location );
         if ($check eq 'skip') {
-            no warnings 'uninitialized';
-            my $msg = "Warning: ".$self->location." skipping value $value because of the following errors:\n"
-                . $self->error_msg . "\n\n";
-            if (not $silent and $msg) {
+            if (not $silent and $self->has_error) {
+                my $msg = "Warning: ".$self->location." skipping value $value because of the following errors:\n"
+                    . $self->error_msg . "\n\n";
                 # fuse UI exits when a warning is issued. No other need to advertise this option
                 print $msg if $args{say_dont_warn};
-                if ($::_use_log4perl_to_warn) {
-                    $user_logger->warn($msg) unless $args{say_dont_warn};
-                }
-                else {
-                    warn $msg unless $args{say_dont_warn};
-                }
+                $user_logger->warn($msg) unless $args{say_dont_warn};
             }
         }
         else {
@@ -1421,6 +1433,7 @@ sub _store {
     }
 
     $logger->trace( "_store done on ", $self->composite_name ) if $logger->is_trace;
+    return;
 }
 
 #
@@ -1441,15 +1454,15 @@ sub transform_boolean {
     }
 
     # convert yes no to 1 or 0
-    $$v_ref = 1 if ( $$v_ref =~ /^y/i or $$v_ref =~ /true/i );
-    $$v_ref = 0 if ( $$v_ref =~ /^n/i or $$v_ref =~ /false/i or length($$v_ref) == 0);
+    $$v_ref = 1 if ( $$v_ref =~ /^(y|yes|true|on)$/i );
+    $$v_ref = 0 if ( $$v_ref =~ /^(n|no|false|off)$/i or length($$v_ref) == 0);
+    return;
 }
 
 # internal. return ( undef, value)
 # May return an undef value if actual store should be skipped
-sub transform_value {
-    my $self  = shift;
-    my %args  = @_ > 1 ? @_ : ( value => $_[0] );
+sub transform_value ($self, @args) {
+    my %args = @args > 1 ? @args : ( value => $args[0] );
     my $value = $args{value};
     my $check = $args{check} || 'yes';
 
@@ -1512,10 +1525,7 @@ sub apply_replace {
     return $value;
 }
 
-sub check_stored_value {
-    my $self = shift;
-    my %args = @_;
-
+sub check_stored_value ($self, %args) {
     my ($ok, $fixed_value) = $self->check_value( %args );
 
     my ( $value, $check, $silent ) =
@@ -1544,12 +1554,11 @@ sub _value_type_error {
     my $str = "Item " . $self->{element_name} . " is not available. " . $self->warp_error;
 
     Config::Model::Exception::User->throw( object => $self, message => $str );
+    return;
 }
 
-sub load_data {
-    my $self = shift;
-
-    my %args  = @_ > 1 ? @_ : ( data => shift );
+sub load_data ($self, @args) {
+    my %args = @args > 1 ? @args : ( data => $args[0] );
     my $data  = delete $args{data} // delete $args{value};
 
     my $rd = ref $data;
@@ -1568,6 +1577,7 @@ sub load_data {
         }
         return $self->store(%args, value => $data);
     }
+    return;
 }
 
 sub fetch_custom {
@@ -1597,6 +1607,7 @@ sub _init {
         $self->submit_to_refer_to;
         $self->{ref_object}->get_choice_from_referred_to;
     }
+    return;
 }
 
 # returns something that needs to be written to config file
@@ -1698,7 +1709,7 @@ sub _fetch {
         :                                      $self->{upstream_default};
     my $std = defined $pref ? $pref : $known_upstream;
 
-    if ( not defined $data and defined $self->{_migrate_from} ) {
+    if ( defined $self->{_migrate_from} and not defined $data ) {
         $data = $self->migrate_value;
     }
 
@@ -1713,6 +1724,7 @@ sub _fetch {
     }
 
     if ( $mode eq 'custom' ) {
+        ## no critic (TestingAndDebugging::ProhibitNoWarning)
         no warnings "uninitialized";
         my $cust;
         $cust = $data
@@ -1725,6 +1737,7 @@ sub _fetch {
     }
 
     if ( $mode eq 'non_upstream_default' ) {
+        ## no critic (TestingAndDebugging::ProhibitNoWarning)
         no warnings "uninitialized";
         my $nbu;
         foreach my $d ($data, $layer_data, $pref) {
@@ -1739,22 +1752,22 @@ sub _fetch {
         return $nbu;
     }
 
-    my $res =
-          $mode eq 'preset'           ? $self->{preset}
-        : $mode eq 'default'          ? $self->{default}
-        : $mode eq 'standard'         ? $std
-        : $mode eq 'layered'          ? $self->{layered}
-        : $mode eq 'upstream_default' ? $self->{upstream_default}
-        : $mode eq 'user'             ? defined $data
-            ? $data
-            : $std
-        : $mode eq 'allow_undef' ? defined $data
-            ? $data
-            : $std
-        : $mode eq 'backend' ? defined $data
-            ? $data
-            : $pref
-        : die "unexpected mode $mode ";
+    my $res;
+    if (any {$_ eq $mode} qw/preset default upstream_default layered/) {
+        $res = $self->{$mode};
+    }
+    elsif ( $mode eq 'standard') {
+        $res = $std;
+    }
+    elsif ( $mode eq 'backend') {
+        $res = $self->_data_or_alt($data, $pref);
+    }
+    elsif (any {$mode eq $_} qw/user allow_undef/) {
+        $res = $self->_data_or_alt($data, $std);
+    }
+    else {
+        die "unexpected mode $mode ";
+    }
 
     $logger->debug( "done in '$mode' mode for " . $self->location . " -> " . ( $res // '<undef>' ) )
         if $logger->is_debug;
@@ -1762,10 +1775,25 @@ sub _fetch {
     return $res;
 }
 
+sub _data_or_alt ($self, $data, $alt) {
+    my $res;
+    my $vt = $self->value_type;
+
+    if (any {$_ eq $vt} qw/integer boolean number/) {
+        $res = $data // $alt
+    }
+    else {
+        # empty string is considered as undef, but empty string is
+        # still returned if there's not defined alternative ($alt)
+        $res = length($data) ? $data : $alt // $data
+    }
+    return $res;
+}
+
 sub fetch_no_check {
     my $self = shift;
     carp "fetch_no_check is deprecated. Use fetch (check => 'no')";
-    $self->fetch( check => 'no' );
+    return $self->fetch( check => 'no' );
 }
 
 # likewise but without any warp, etc related check
@@ -1779,18 +1807,15 @@ sub _fetch_no_check {
         :                                  $self->{default};
 }
 
-sub fetch_summary {
-    my $self = shift;
-    my $value = $self->fetch(@_) // '<undef>';
+sub fetch_summary ($self, @args) {
+    my $value = $self->fetch(@args) // '<undef>';
     $value =~ s/\n/ /g;
     $value = substr( $value, 0, 15 ) . '...' if length($value) > 15;
     return $value;
 }
 
-sub fetch {
-    my $self = shift;
-
-    my %args = @_ > 1 ? @_ : ( mode => $_[0] );
+sub fetch ($self, @args) {
+    my %args = @args > 1 ? @args : ( mode => $args[0] );
     my $mode   = $args{mode}   || 'backend';
     my $silent = $args{silent} || 0;
     my $check  = $self->_check_check( $args{check} );
@@ -1846,14 +1871,8 @@ sub fetch {
     elsif ( $check eq 'skip' ) {
         my $msg = $self->error_msg;
         my $str = $value // '<undef>';
-        if ($::_use_log4perl_to_warn) {
-            $user_logger->warn("Warning: fetch [".$self->name,"] skipping value $str because of the following errors:\n$msg\n")
+        $user_logger->warn("Warning: fetch [".$self->name,"] skipping value $str because of the following errors:\n$msg\n")
             if not $silent and $msg;
-        }
-        else {
-            warn "Warning: fetch [".$self->name,"] skipping value $str because of the following errors:\n$msg\n\n"
-                if not $silent and $msg;
-        }
         # this method is supposed to return a scalar
         return undef; ## no critic(Subroutines::ProhibitExplicitReturnUndef)
 
@@ -1867,32 +1886,19 @@ sub fetch {
     return;
 }
 
-sub map_write_as {
-    my $self = shift;
+sub map_write_as ($self, @args) {
     my @res;
     if ($self->{write_as} and $self->value_type eq 'boolean') {
-        foreach my $v (@_) {
+        foreach my $v (@args) {
             push @res, ( defined $v and $v =~ /^\d+$/ ) ? $self->{write_as}[$v] : $v;
         }
     }
     else {
-        @res = @_;
+        @res = @args;
     }
     return wantarray ? @res : $res[0];
 }
 
-# modifies in place the passed values
-sub map_write_as_inplace {
-    my $self = shift;
-    return unless $self->{write_as} and $self->value_type eq 'boolean';
-
-    foreach (@_) {
-        if (defined $_ and /^\d+$/) {
-            $_ = $self->{write_as}[$_];
-        }
-    }
-}
-
 sub user_value {
     return shift->{data};
 }
@@ -1905,6 +1911,7 @@ sub fetch_preset {
 sub clear {
     my $self = shift;
     $self->store(undef);
+    return;
 }
 
 sub clear_preset {
@@ -1924,9 +1931,8 @@ sub clear_layered {
     return defined $self->{preset} || defined $self->{data};
 }
 
-sub get {
-    my $self = shift;
-    my %args = @_ > 1 ? @_ : ( path => $_[0] );
+sub get ($self, @args) {
+    my %args = @args > 1 ? @args : ( path => $args[0] );
     my $path = delete $args{path};
     if ($path) {
         Config::Model::Exception::User->throw(
@@ -1937,16 +1943,14 @@ sub get {
     return $self->fetch(%args);
 }
 
-sub set {
-    my $self = shift;
-    my $path = shift;
+sub set ($self, $path, @data) {
     if ($path) {
         Config::Model::Exception::User->throw(
             object  => $self,
             message => "set() called with a value with non-empty path: '$path'"
         );
     }
-    return $self->store(@_);
+    return $self->store(@data);
 }
 
 #These methods are important when this leaf value is used as a warp
@@ -1963,6 +1967,7 @@ sub register_dependency {
     # weaken only applies to the passed reference, and there's no way
     # to duplicate a weak ref. Only a strong ref is created.
     weaken( $self->{depend_on_me}[0] );
+    return;
 }
 
 sub get_depend_slave {
@@ -1996,7 +2001,7 @@ Config::Model::Value - Strongly typed configuration value
 
 =head1 VERSION
 
-version 2.142
+version 2.153
 
 =head1 SYNOPSIS
 
@@ -2070,7 +2075,8 @@ purpose. (C<upstream_default> parameter)
 =item *
 
 mandatory value: reading a mandatory value raises an exception if the
-value is not specified and has no default value.
+value is not specified (i.e is C<undef> or empty string) and has no
+default value.
 
 =item *
 
@@ -3095,7 +3101,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2021 by Dominique Dumont.
+This software is Copyright (c) 2005-2022 by Dominique Dumont.
 
 This is free software, licensed under:
 
